<?php

//クローラー判定
$userAgent = $_SERVER['HTTP_USER_AGENT'] ?? "";
$isCrawler = preg_match('/bot|crawl|slurp|spider/i', $userAgent);
if ($isCrawler) {
    //とりあえず成功ということにしとく
    echo '{"status":"success"}';
    exit();
}

//共通処理呼び出し
$require_login = false; //ログイン不要
$no_nsfw_check = true; //NSFWチェック不要
$no_entry_check = true; //エントリーチェック不要
require_once("../common.php");

$ca = new CommentApi($db, $SETTING);
$result = $ca->Run($LOGIN);
echo $result;


/*-----------------------------------------------------------------*/
class CommentApi
{
    private $db;
    private $setting;
    public function __construct($db, $setting)
    {
        $this->db = $db;
        $this->setting = $setting;
    }

    public function Run($login)
    {
        $forms = $this->FilterInput();
        $mode = $forms['mode'];
        try {
            if ($mode == 'post') {
                return $this->Post($forms);
            } elseif ($mode == 'adminpost') {
                return $this->AdminPost($forms, $login);
            } elseif ($mode == 'delete') {
                return $this->Delete($forms, $login);
            } elseif ($mode == 'approved') {
                return $this->Approved($forms, $login);
            } elseif ($mode == "get") {
                return $this->GetCommentList($forms, $login);
            } elseif ($mode == "check") {
                return $this->checkUnreadComment($forms, $login);
            } elseif ($mode == "read") {
                return $this->ReadSet($forms, $login);
            } else {
                return "{\"status\":\"error\", \"message\":\"コマンドが不正です\"}";
            }
        } catch (Exception $e) {
            $msg = $e->getMessage();
            return "{\"status\":\"error\", \"message\":\"{$msg}\"}";
        }
    }

    public function FilterInput()
    {
        $forms = array();
        $forms['mode'] =  htmlspecialchars(filter_input(INPUT_POST, 'mode') ?? filter_input(INPUT_GET, 'mode') ?? "");

        $forms['target'] = htmlspecialchars(filter_input(INPUT_POST, 'target') ?? -1);

        $forms['article_id'] = filter_input(INPUT_POST, 'article_id', FILTER_SANITIZE_NUMBER_INT) ?? filter_input(INPUT_GET, 'article_id', FILTER_SANITIZE_NUMBER_INT) ?? -1;
        if ($forms['article_id'] === null || $forms['article_id'] === "") {
            $forms['article_id'] = -1;
        }
        $forms['parent_id'] = filter_input(INPUT_POST, 'parent_id', FILTER_SANITIZE_NUMBER_INT) ?? filter_input(INPUT_GET, 'article_id', FILTER_SANITIZE_NUMBER_INT) ?? -1;
        if ($forms['parent_id'] === null || $forms['parent_id'] === "") {
            $forms['parent_id'] = -1;
        }
        $forms['name'] =  htmlspecialchars(filter_input(INPUT_POST, 'name') ?? "");
        $forms['email'] =  htmlspecialchars(filter_input(INPUT_POST, 'email') ?? "");
        $forms['comment'] = htmlspecialchars(filter_input(INPUT_POST, 'comment') ?? "");

        $forms['is_approved'] =  filter_input(INPUT_POST, 'approved', FILTER_VALIDATE_BOOL) ?? 0;

        $forms['show_unapproved'] = filter_input(INPUT_GET, 'show_unapproved', FILTER_VALIDATE_BOOL) ?? false;
        $forms['count'] =  filter_input(INPUT_GET, 'count', FILTER_VALIDATE_INT) ?? 20;
        $forms['page'] =  filter_input(INPUT_GET, 'index', FILTER_VALIDATE_INT) ?? 0;

        $forms['success_message'] = htmlspecialchars(filter_input(INPUT_POST, 'return_message') ?? 'コメントを受け付けました。');
        $forms['title'] = filter_input(INPUT_POST, 'title');

        return $forms;
    }

    private function Validate($forms)
    {
        //自動入力排除用
        $title = $forms["title"];
        if ($title != "") {
            die('{"status":"error","message":"送信できません"}');
        }

        //送信元確認用
        $from = $_SERVER['HTTP_REFERER'] ?? "";
        if ($from == "") {
            die('{"status":"error","message":"送信できません"}');
        }
    }

    public function Post($forms)
    {
        //入力値チェック
        $this->Validate($forms);
        //プラグインによるデータチェック
        $plugin = "";
        if (file_exists("../env/plugin/plugin_comment_validate.php")) {
            $plugin = "../env/plugin/plugin_comment_validate.php";
        } else if (file_exists("../plugin/plugin_comment_validate.php")) {
            $plugin = "../plugin/plugin_comment_validate.php";
        }
        if ($plugin != "") {
            require_once($plugin);
            $plugin_func = "plugin_comment_validate\\validate";
            $plugin_func($forms);
        }

        if ($forms['comment'] == "") {
            throw new \Exception("コメント内容を入力してください");
        }

        try {
            $comment_cls = new ppp\Comments($this->db);
            $comment_cls->addComment($forms['target'], $forms['name'], $forms['email'], $forms['comment'], $forms['parent_id'], false);
        } catch (Exception $e) {
            //失敗
            $msg = $e->getMessage();
            return "{\"status\":\"error\", \"message\":\"$msg\"}";
        }

        $msg = $forms['success_message'];
        if ($forms['target'] != -1 && $this->setting['use_comment_approved']) {
            $msg = $msg . "<br>コメントは承認後に表示されます。";
        }
        return "{\"status\":\"success\", \"message\":\"$msg\"}";
    }

    public function AdminPost($forms, $login)
    {
        if (!$login) {
            return "{\"status\":\"error\", \"message\":\"ログインしていません\"}";
        }

        //入力値チェック
        if ($forms['comment'] == "") {
            throw new \Exception("コメント内容を入力してください");
        }

        try {
            $comment_cls = new ppp\Comments($this->db);
            $comment_cls->addComment($forms['article_id'], $forms['name'], $forms['email'], $forms['comment'], $forms['parent_id'], true);
        } catch (Exception $e) {
            //失敗
            $msg = $e->getMessage();
            return "{\"status\":\"error\", \"message\":\"$msg\"}";
        }

        $msg = "コメントを受け付けました";
        return "{\"status\":\"success\", \"message\":\"$msg\"}";
    }

    public function Approved($forms, $login)
    {
        if (!$login) {
            return "{\"status\":\"error\", \"message\":\"ログインしていません\"}";
        }

        //入力値チェック
        if ($forms['target'] == null) {
            throw new \Exception("対象を選択してください");
        }

        try {
            $comment_cls = new ppp\Comments($this->db);
            $comment_cls->approveComment($forms['target']);
        } catch (Exception $e) {
            //失敗
            $msg = $e->getMessage();
            return "{\"status\":\"error\", \"message\":\"$msg\"}";
        }
        return "{\"status\":\"success\", \"message\":\"\"}";
    }

    public function ReadSet($forms, $login)
    {
        if (!$login) {
            return "{\"status\":\"error\", \"message\":\"ログインしていません\"}";
        }

        //入力値チェック
        if ($forms['target'] == null) {
            throw new \Exception("対象を選択してください");
        }

        try {
            $comment_cls = new ppp\Comments($this->db);
            $comment_cls->readComment($forms['target']);
        } catch (Exception $e) {
            //失敗
            $msg = $e->getMessage();
            return "{\"status\":\"error\", \"message\":\"$msg\"}";
        }
        return "{\"status\":\"success\", \"message\":\"\"}";
    }

    public function Delete($forms, $login)
    {
        if (!$login) {
            return "{\"status\":\"error\", \"message\":\"ログインしていません\"}";
        }

        //入力値チェック
        if ($forms['target'] == null) {
            throw new \Exception("対象を選択してください");
        }

        try {
            $comment_cls = new ppp\Comments($this->db);
            $comment_cls->deleteComment($forms['target']);
        } catch (Exception $e) {
            //失敗
            $msg = $e->getMessage();
            return "{\"status\":\"error\", \"message\":\"$msg\"}";
        }
        return "{\"status\":\"success\", \"message\":\"\"}";
    }
    public function CheckUnreadComment($forms, $login)
    {
        $article_id = $forms["article_id"];
        if ($article_id == -1 || $article_id == "") {
            if (!$login) {
                return "{\"status\":\"error\", \"message\":\"ログインしていません\"}";
            }
        }
        $comment_cls = new ppp\Comments($this->db);
        $ret = $comment_cls->getUnreadCommentCount("article_id=:article_id", ["article_id" => $article_id]);
        return "{\"status\":\"success\", \"message\":\"$ret\"}";
    }
    public function GetCommentList($forms, $login)
    {
        $article_id = $forms["article_id"];
        $count = $forms["count"];
        $page = $forms["page"];
        $show_unapproved = $forms["show_unapproved"];

        if ($show_unapproved) {
            if (!$login) {
                return "{\"status\":\"error\", \"message\":\"ログインしていません\"}";
            }
        }
        if ($article_id == -1 || $article_id == "") {
            if (!$login) {
                return "{\"status\":\"error\", \"message\":\"ログインしていません\"}";
            }
        }
        $comment_cls = new ppp\Comments($this->db);
        $comments = $comment_cls->getCommentsByArticleId($article_id, $show_unapproved);

        //UIレンダリング
        if (file_exists('../env/templates/template_article_comment_tree.php')) {
            //ユーザーテンプレートがある場合はそちらを採用する
            $template = new NoSixTemplate('template_article_comment_tree.php', '../env/templates/');
        } else {
            $template = new NoSixTemplate('template_article_comment_tree.php', '../templates');
        }
        $template->set_Data('comments', $comments);
        $template->set_Data('article_id', $article_id);
        ob_start();
        $template->display();
        $output = ob_get_contents();
        ob_end_clean();

        return $output;
    }
}
