<?php

/********************************************************************
 *
 * 全ページ共通処理と関数定義
 *
 * 定義される共通変数
 * $VERSION ... 現在のシステムバージョン
 * $DEBUG ... Trueの場合デバッグモード
 * $DEMO_MODE ... Trueの場合デモモード
 * $PATH_INFO ... パス関連に関する情報
 * $LOGIN ... Trueの場合ログイン済み
 * $db ... dbへの接続。必ず呼び出し元でcloseすること
 * $SETTING　... settingテーブルの内容
 * $NSFW_CONSENT　... NSFW同意が取れているか。trueの場合取れている
 *
 * 入力される共通変数
 * $require_login ... trueの場合、ログインチェックが行われ、ログインしていない場合ログインページにリダイレクトする
 *
 ********************************************************************/

$VERSION = "1.24.0";

$DEBUG = false;
if (strstr($_SERVER['HTTP_HOST'], "127.0.0.1") || strstr($_SERVER['HTTP_HOST'], "192.168.")) {
    ini_set('display_errors', "On"); //ローカルテスト時は警告有効
    $DEBUG = true;
} else {
    ini_set('display_errors', "Off");
}

//パス関係取得
$PATH_INFO = getPathInfo();
$rel = $PATH_INFO["system_root_rel"]; //common.phpを呼び出したURLからシステムルートまでの相対パス
if ($rel == "./") {
    $redirect_rel = "";
} else {
    $redirect_rel = $rel;
}


//呼び出し元URL
$BACK_URL = filter_input(INPUT_GET, 'back_url') ?? filter_input(INPUT_POST, 'back_url') ?? "";
if ($BACK_URL == "") {
    $BACK_URL = $_SERVER['HTTP_REFERER'] ?? "";
    if (!strstr($BACK_URL, $PATH_INFO["url_system_root"])) {
        //外部URLならトップページを戻り先にする
        $BACK_URL = $redirect_rel;
    }
}

//デモモード動作フラグ。trueにするとHTML投稿が無効になる。
$DEMO_MODE = false;
if (file_exists("$rel/demo.php")) {
    $DEMO_MODE = true;
}

//システム設定ファイルの存在確認
$SYSTEM_SETTING_FILE = "{$rel}env/system_setting.php";
if (!file_exists($SYSTEM_SETTING_FILE)) {
    //存在しない場合、初回起動なのでシステム初期化画面へ
    header("Location: {$redirect_rel}initialize.php");
    exit;
}

//ログイン有無確認
$LOGIN = false;
if (file_exists("{$rel}env/tokens.php")) {
    require_once("{$rel}token.php");
    $LOGIN = checkToken($rel);
}
if ($require_login && !$LOGIN) {
    //ログイン必須でログインしていない場合ログイン画面に移動
    header("Location: {$redirect_rel}login.php?request=" . $PATH_INFO["system_request"]);
    exit();
}

//DB接続
require_once("{$rel}env/system_setting.php");
$db = new SQLite3("{$rel}$system_db_file");
$db->enableExceptions(true); //例外発生をONに設定
$db->busyTimeout(5000); //ビジーハンドルを5秒に設定

//設定値読み取り
readSetting();

//always_on_ssl確認
if ($SETTING['always_on_ssl'] == 1) {
    //httpsにリダイレクトする
    if (empty($_SERVER['HTTPS']) && !strstr($_SERVER['HTTP_HOST'], "127.0.0.1") && !strstr($_SERVER['HTTP_HOST'], "192.168.")) {
        $db->close();
        header("Location: https://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}");
        exit;
    }
}

//NSFWチェック
$NSFW_CONSENT = "0";
if (!isset($no_nsfw_check)) {
    //Cookie確認
    $NSFW_CONSENT = filter_input(INPUT_COOKIE, 'nsfw_consent', FILTER_VALIDATE_BOOLEAN) ?? 0;
    //NSFWサイトの場合同意が必要
    if ($SETTING['nsfw']) {
        if (!$NSFW_CONSENT) {
            //まだ同意を得ていない場合は確認画面に移動
            header("Location: {$redirect_rel}nsfw.php");
        }
    }
}

//エントリーチェック
$ENTRY_CHECK = true;
if (!isset($no_entry_check) && !$LOGIN) {
    //EntryPage使用
    if ($SETTING['use_entry']) {
        $ENTRY_CHECK = filter_input(INPUT_COOKIE, 'entry_consent') ?? false;
        if ($ENTRY_CHECK) {
            //EntryPasswordの正当性をチェック
            if ($SETTING['entry_password']) {
                $cookie_password = filter_input(INPUT_COOKIE, 'entry_password') ?? "";
                $check_pass = $SETTING['entry_password'];
                if (!password_verify($check_pass, $cookie_password)) {
                    $ENTRY_CHECK = false;
                }
            }
        }
        if (!$ENTRY_CHECK) {
            //まだ同意を得ていない場合は確認画面に移動
            header("Location: {$redirect_rel}entry.php");
        }
    }
}

//オートローダーを登録
spl_autoload_register(function ($class) {
    global $PATH_INFO;
    $baseDir = $PATH_INFO["system_root_rel"];
    $prefix = 'ppp\\';
    $len = strlen($prefix);
    if (strncmp($prefix, $class, $len) === 0) {
        $relativeClass = strtolower(substr($class, $len));
        $file = $baseDir . '/class/' . str_replace('\\', '/', $relativeClass) . '.php';

        if (file_exists($file)) {
            require $file;
        }
    } else {
        $file = $baseDir . '/module/' . strtolower(str_replace('\\', '/', $class)) . '.php';
        if (file_exists($file)) {
            require $file;
        }
    }
});

///////////////// 共通利用function /////////////////
function readSetting()
{
    global $db;
    global $SETTING;
    global $DEMO_MODE;

    //設定値の読み取り / 出力する可能性のある項目についてサニタイジング
    try {
        $SETTING = $db->querySingle('SELECT * FROM setting', true);
    } catch (Exception $ex) {
        echo "システムエラーが発生しました。時間をおいて再度アクセスしてみてください。<br>";
        exit;
    }
    $SETTING["site_title"] = h($SETTING["site_title"]);
    $SETTING["display_name"] = h($SETTING["display_name"]);
    $SETTING["twitter_card_text"] = h($SETTING["twitter_card_text"]);
    $SETTING["image_color"] = h($SETTING["image_color"]);

    //トップページの表示内容をフォーマット
    if (!$SETTING['instruction_text_use_html']) {
        //HTMLを使用しないモードの場合はHTMLタグを取り除く
        $SETTING['instruction_text'] = strip_tags($SETTING['instruction_text']);
    }
    //改行を<BR>に変換
    $SETTING['instruction_text'] = convertLineBreaks($SETTING['instruction_text']);
    $SETTING['guest_attention'] = convertLineBreaks($SETTING['guest_attention']);

    //バナー
    $SETTING['banner'] = '';
    if ($SETTING['use_mini_banner'] ?? 0) {
        $SETTING['banner'] = '<img src="./images/mini_banner.png" width="200" height="40" id="mini_banner">';
    }
}
function debug($arg)
{
    echo "<pre>";
    print_r($arg);
    echo "</pre>";
}

function debug_f(...$args)
{
    global $DEBUG;

    if ($DEBUG) {
        $debugFilePath = './debug.txt';
        $data = date('Y-m-d H:i:s') . ' ' . implode(' ', $args) . PHP_EOL;

        if (file_put_contents($debugFilePath, $data, FILE_APPEND | LOCK_EX) === false) {
            echo 'Failed to write to debug file.';
        }
    }
}
function h($val)
{
    return htmlspecialchars($val ?? "", ENT_QUOTES, "UTF-8");
}


function getPathInfo()
{
    $result = array();

    $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http';
    $host = $_SERVER['HTTP_HOST'];
    $url = $host . $_SERVER['SCRIPT_NAME'];
    $request = $_SERVER['REQUEST_URI'];
    //$request = filter_input(INPUT_SERVER, 'REQUEST_URI', FILTER_SANITIZE_URL);
    $request = urldecode($request);
    $request = $protocol . '://' . $host . $request;

    $url_dir = dirname($url);
    if (substr($url_dir, -1) == '/') {
        $url_dir = substr($url_dir, 0, -1);
    }
    //URL中、/が連続しているケースがあるので取り除く
    $url = preg_replace('#/+#', '/', $url);
    $url_dir = preg_replace('#/+#', '/', $url_dir);
    $url = $protocol . '://' . $url;
    $url_dir = $protocol . '://' . $url_dir;

    if (strpos($url, "/api/")) {
        $url_system_root = dirname($url_dir); //APIディレクトリ内なら1階層上
        $rel = "../";
    } else {
        $url_system_root = $url_dir;
        $rel = "./";
    }

    $result["url_dir"] = $url_dir; //呼び出し元ファイルのディレクトリまでのURL
    $result["url_system_root"] = $url_system_root; //このシステムのルートディレクトリ
    $result["system_root_rel"] = $rel; //呼び出し元からシステムルートへの相対パス

    $tmp = str_replace($url_system_root . "/", "", $request);
    if (substr($tmp, 0, 1) == "/") {
        $tmp = '.' . $tmp;
    }
    $result["system_request"] = $tmp; //システムルートから見たリクエストURL

    return $result;
}

//最大アップロード可能ファイルサイズ取得(バイト単位)
function getMaxUploadSize()
{
    $limit_max = (ini_get_bytes('post_max_size') < ini_get_bytes('upload_max_filesize')) ? 'post_max_size' : 'upload_max_filesize';
    $limit_max = (ini_get_bytes($limit_max) < ini_get_bytes('memory_limit')) ?  $limit_max : 'memory_limit';

    return ini_get_bytes($limit_max);
}

//設定オプションを整数表現に変換して返す
function ini_get_bytes($varname)
{
    $val = ini_get($varname);
    $last = strtolower(substr($val, -1));
    $val = (int)($val);
    switch ($last) {
        case 'g':
            $val *= 1024;
            // no break
        case 'm':
            $val *= 1024;
            // no break
        case 'k':
            $val *= 1024;
    }

    return $val;
}

//文字列の中の改行コードを<br>に変換する
//ただし、HTMLブロック要素の開始・終了直後及び<script>/<style>内の改行コードは変換しない
function convertLineBreaks($input)
{
    $output = $input;

    // Remove newlines after block elements
    $blockElements = array(
        'address',
        'article',
        'aside',
        'blockquote',
        'canvas',
        'dd',
        'div',
        'dl',
        'dt',
        'fieldset',
        'figcaption',
        'figure',
        'footer',
        'form',
        'h1',
        'h2',
        'h3',
        'h4',
        'h5',
        'h6',
        'header',
        'hr',
        'li',
        'main',
        'nav',
        'noscript',
        'ol',
        'p',
        'pre',
        'section',
        'table',
        'tbody',
        'tr',
        'td',
        'th',
        'tfoot',
        'ul',
        'video',
        'style',
        'script'
    );

    $output = preg_replace('/\r/', '', $output);


    foreach ($blockElements as $blockElement) {
        $output = preg_replace('/(<\/?' . $blockElement .  '[^>]*?>)\n/i', '$1', $output);
    }

    $output = preg_replace_callback('/<script[^>]*?>(.*)<\/script>/is', function ($matches) {
        return preg_replace('/\n/', '[[BR]]', $matches[0]);
    }, $output);

    $output = preg_replace_callback('/<style[^>]*?>(.*)<\/style>/is', function ($matches) {
        return preg_replace('/\n/', '[[BR]]', $matches[0]);
    }, $output);
    $output = preg_replace('/^\n/m', '<br>', $output); //空行を<br>に変換
    $output = preg_replace('/(\{\/?\.\.[^\}]*?\})\n/i', '$1', $output); // {..***}～{/..}表記直後の改行を無効化
    $output = preg_replace('/\n/', '<br>', $output);

    $output = preg_replace('/\[\[BR\]\]/', "\r\n", $output);

    return $output;
}

function getTemplates()
{
    global $db;

    //テンプレートを読み込み
    try {
        // クエリを実行して結果を取得
        $results = $db->query('SELECT * FROM templates');

        // 結果を配列に格納
        $templates = array();
        while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
            $key = $row['template_name'];
            unset($row['id']);
            unset($row['template_name']);
            $row['template'] = $row['template'];
            $templates[$key] = $row;
        }
        return $templates;
    } catch (Exception $ex) {
        echo "システムエラーが発生しました。時間をおいて再度アクセスしてみてください。<br>";
        exit;
    }
}
