<?php

/********************************************************************
 *
 * 閲覧用 記事一覧ページ
 *
 *  GET q   : 記事絞込用の検索ワード。スペース区切りで複数指定するとAND検索。
 *      page: ページネーション時の表示対象ページ番号。
 *
 ********************************************************************/


//共通処理呼び出し
$require_login = false; //ログイン不要
require_once("./common.php");

$records_per_page = 15; //1ページ内の表示件数

$searchs =  filter_input(INPUT_GET, 'q');
$index =  filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT) ?? 1;
$order =  filter_input(INPUT_GET, 'order') ?? 1;
$comment =  filter_input(INPUT_GET, 'comment') ?? 1;
$static_mode =  filter_input(INPUT_GET, 'static_mode') ?? "";
$limit = $records_per_page;
$offset = 0;
if ($index != "") {
    $offset = ($index - 1)  * $records_per_page;
}

$row_count = 0;
try {
    // SELECT 文を実行してデータを取得
    if ($searchs == "") {
        //検索条件指定なし
        $row_count = $db->querySingle('SELECT Count(id) as count From articles Where hidden != 1');
        $pages = ceil($row_count / $records_per_page);

        $result = $db->query('SELECT id, created_at, title, comment, tags, nsfw, alt_url From articles Where hidden != 1 ORDER BY created_at DESC LIMIT ' . $records_per_page . ' OFFSET ' . $offset);
    } else {
        //検索条件指定あり
        $search_list = '';
        $search_idx = 0;
        $search_and = '';
        foreach (explode(' ', $searchs) as $search) {
            if (substr($search, 0, 1) == "#") {
                $search_list = $search_list .  $search_and . "(tags like :search{$search_idx} or tags like :search_m{$search_idx})";
            } else {
                $search_list = $search_list .  $search_and . "(title like :search{$search_idx} or comment like :search{$search_idx} or tags like :search{$search_idx})";
            }
            $search_and = ' and ';
            $search_idx++;
        }
        $stmt = $db->prepare('SELECT Count(id) as count From articles Where hidden != 1 AND  ' . $search_list);
        $search_idx = 0;
        foreach (explode(' ', $searchs) as $search) {
            if (substr($search, 0, 1) == "#") {
                $stmt->bindValue(":search{$search_idx}", '% ' . substr($search, 1) . ' %');
                $stmt->bindValue(":search_m{$search_idx}", '% -' . substr($search, 1) . ' %');
            } else {
                $stmt->bindValue(":search{$search_idx}", '%' . $search . '%');
            }
            $search_idx++;
        }
        $result = $stmt->execute();
        $row = $result->fetchArray();
        $row_count = $row['count'];
        $pages = ceil($row_count / $records_per_page);

        $stmt = $db->prepare('SELECT id, created_at, title, comment, tags, nsfw, alt_url, thumbnail From articles Where hidden != 1 AND ' . $search_list . ' ORDER BY created_at DESC LIMIT ' . $records_per_page . ' OFFSET ' . $offset);
        $search_idx = 0;
        foreach (explode(' ', $searchs) as $search) {
            if (substr($search, 0, 1) == "#") {
                $stmt->bindValue(":search{$search_idx}", '% ' . substr($search, 1) . ' %');
                $stmt->bindValue(":search_m{$search_idx}", '% -' . substr($search, 1) . ' %');
            } else {
                $stmt->bindValue(":search{$search_idx}", '%' . $search . '%');
            }
            $search_idx++;
        }
        $result = $stmt->execute();
    }

    $idx = 0;
    $list_html = "";
    while ($row = $result->fetchArray()) {
        $created_at = h($row['created_at']);
        $title = h($row['title']);
        $id = h($row['id']);
        $alt_url = h($row['alt_url'] ?? "");
        $comment = $row['comment'];
        $nsfw = $row['nsfw'] ?? false;
        if ($comment) {
            preg_match('/{caption}(.*?){\/caption}/si', $comment, $match);
            if ($match) {
                //キャプションがある場合はそれを使用
                $comment = $match[1];
            }

            //特殊記述の削除
            $comment = preg_replace('/\{\{/si', '&#123;', $comment); //特殊記述の削除
            $comment = preg_replace('/\}\}/si', '&#125;', $comment); //特殊記述の削除
            $comment = preg_replace('/\{.*\}/si', '', $comment);
            $comment = preg_replace('/\r\n/', ' ', $comment);
            $comment = preg_replace('/\n/', ' ', $comment);
            $comment = strip_tags($comment);

            // 一部マークダウンを削除する
            $comment = preg_replace('/^(#+)\s*(.*)/m', "$2", $comment);
            $comment = preg_replace('/\*\*(.*)\*\*/', "$1", $comment);
            $comment = preg_replace('/\*(.*)\*/', "$1", $comment);

            $comment = mb_substr(strip_tags($comment), 0, 100);
        }

        if (($nsfw && !$NSFW_CONSENT) || ($nsfw && $SETTING["always_nsfw_check"])) {
            //nsfw同意が取れていない
            $img = "<div class='list-thumb-box background-nsfw'></div>";
        } else {
            $thumbs = $db->query('SELECT * FROM images where article_id = ' . $id . ' order by sort asc');
            $thumbsarray = array();
            $tidx = 0;
            $trow = $thumbs->fetchArray();
            $img = "";
            if ($trow) {
                $file_name = h($trow['thumb_name']);
                $img = "<div class='list-thumb-box' style=\"background-image:url('./images/$file_name')\"></div>";
            } else {
                //画像なし
                if ($row['thumbnail'] != "" && $row['thumbnail'] != "@text" && $row['thumbnail'] != "@user_thumbnail" && $row['thumbnail'] != "@all") {
                    $html_thumb = "./env/thumbnails/{$row['thumbnail']}";
                    $html_thumb_background = "style=\"background-image:url('$html_thumb')\"";
                } else {
                    $html_thumb_background = "";
                }
                $img = "<div class='list-thumb-text' $html_thumb_background></div>";
            }
        }
        if ($SETTING["use_rewrite"]) {
            $url = "./log/{$id}";
            if ($alt_url) {
                $url = "./" . $alt_url;
            }
        } else {
            $url = "./view.php?id={$id}";
        }
        $list_html = $list_html . "<li><a href='$url'>{$img}<div class='search-list-info'><span class='created_at'>{$created_at}</span><span class='title'>{$title}</span><span class='comment'>{$comment}</span></div></a></li>";
    }
    $db->close();

    $message = "";
    if ($row_count == "0") {
        if ($searchs == "") {
            $message = "<div id='message'>まだ記事は投稿されていません</div>";
        } else {
            $message = "<div id='message'>「 $searchs 」に該当する記事は見つかりません</div>";
        }
    }

    $indexser = "";
    for ($i = 1; $i <= $pages; $i++) {
        if ($i == $index) {
            $indexser = $indexser . '<span class="pager-current-index">' . $i . '</span> ';
        } else {
            if ($searchs == "") {
                $indexser = $indexser . '<a href="?page=' . $i . '">' . $i . '</a> ';
            } else {
                $indexser = $indexser . '<a href="?page=' . $i . '&q=' . urlencode($searchs)  . '">' . $i . '</a> ';
            }
        }
    }
} catch (Exception $e) {
    echo $e->getMessage();
}

//UIレンダリング
require_once('./module/NoSixTemplate.php');
if (file_exists('./env/templates/template_list.php')) {
    //ユーザーテンプレートがある場合はそちらを採用する
    $template = new NoSixTemplate('template_list.php', './env/templates/');
} else {
    $template = new NoSixTemplate('template_list.php', './templates');
}
$template->set_data('setting', $SETTING); //Array
$template->set_data('searchs', $searchs);
$template->set_data('message', $message);
$template->set_data('list_html', $list_html);
$template->set_data('indexser', $indexser);
$template->set_data('login', $LOGIN); //ログイン有無
$template->set_data('static_mode', $static_mode);
$template->set_data('back_url', $BACK_URL);
$template->set_data('VERSION', $VERSION);

if ($static_mode == 1) {
    //静出力モード
    $file_name = "./list.html";
    ob_start();

    $template->display();

    $output = ob_get_contents();
    ob_end_clean();
    file_put_contents($file_name, $output);
    echo ('{"state":"success", "path", "' . $file_name . '"}');
    exit;
}

$template->display();
